@@ -16,9 +16,9 @@
 
 import os
 import sys
-import sudsds
+# import sudsds
+from suds import MethodNotFound
 from calculate.core.server.cert_cmd import getHwAddr, getIpLocal
-
 _ = lambda x: x
 from calculate.lib.cl_lang import setLocalTranslate
 
@@ -77,7 +77,7 @@ def create_obj(client, method_name):
 
     info = operation.methods.get(method_name)
     if not info:
-        raise sudsds.MethodNotFound(method_name)
+        raise MethodNotFound(method_name)
     type_op = info.binding.input.param_defs(info)[param][inf_param]
     str_type = type_op.type[param_type]
 
