@@ -13,13 +13,17 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
+
 from calculate.core.server.loaded_methods import LoadedMethods
 
-from soaplib.serializers.primitive import String, Integer, Boolean, string_encoding
-from soaplib.serializers.clazz import Array, ClassSerializer
-from soaplib.serializers import nillable_value, nillable_element
-from soaplib.serializers import string_to_xml
-from soaplib.service import rpc
+from spyne import String, Integer, Boolean
+from spyne.model.primitive import string_encoding
+from spyne import Array, ComplexModel
+# from spyne.model import Mandatory 
+from spyne import rpc
+
+
+
 import sys
 import pickle
 import os
@@ -30,11 +34,11 @@ _ = lambda x: x
 from calculate.lib.cl_lang import setLocalTranslate
 
 setLocalTranslate('cl_core3', sys.modules[__name__])
-
+String.Attributes.encoding = "utf-8"
 #####
 # API
 #####
-class CommonInfo(ClassSerializer):
+class CommonInfo(ComplexModel):
     """
     Common info for all Info classes
 
@@ -52,24 +56,14 @@ class CommonInfo(ClassSerializer):
 class LazyString(object):
     pass
 
-def lazyable(cl):
-    def to_xml(cls, value, tns, parent_elt, name='retval'):
-        if isinstance(value, LazyString):
-            value = str(value)
-        if not isinstance(value, unicode):
-            value = unicode(value, string_encoding)
 
-        string_to_xml(cls, value, tns, parent_elt, name)
-
-    cl.to_xml = classmethod(nillable_value(to_xml))
-    return cl
-
-String = lazyable(String)
-
-
-class DataVarsSerializer(ClassSerializer):
+class DataVarsSerializer(ComplexModel):
     """Serializer for datavars types"""
 
+    class Attributes(ComplexModel.Attributes):
+        default = None
+        min_occurs = 1
+
     def elementByType(self, typeobj):
         """Get element by variable type, given for table or not"""
         elementMap = {'table': 'table',
@@ -114,10 +108,10 @@ class DataVarsSerializer(ClassSerializer):
 
 
 class ChoiceValue(DataVarsSerializer):
-    typefield = String
-    values = Array(String)
-    comments = Array(String)
-    onChanged = Array(String)
+    typefield = String(default=None, min_occurs=1)
+    values = Array(String, default=None, min_occurs=1)
+    comments = Array(String, default=None, min_occurs=1)
+    onChanged = Array(String, default=None, min_occurs=1)
 
     def __init__(self, dv=None, varObj=None, readOnly=False, **kwargs):
         if dv:
@@ -154,22 +148,25 @@ class ChoiceValue(DataVarsSerializer):
 
 
 class Table(DataVarsSerializer):
-    head = Array(String)
-    fields = Array(String)  # name fields in params object
-    body = Array(Array(String))
-    values = Array(ChoiceValue)
-    onClick = String
-    addAction = String
+    # head = Mandatory(Array(String(default=None, min_occurs=1), min_occurs=1))
+    head = Array(String(default=None, min_occurs=1), min_occurs=1)
+    fields = Array(String(default=None, min_occurs=1), min_occurs=1)  # name fields in params object
+    body = Array(Array(String), default=None, min_occurs=1)
+    values = Array(ChoiceValue, default=None, min_occurs=1)
+    onClick = String(default=None, min_occurs=1)
+    addAction = String(default=None, min_occurs=1)
     # шаг на который нужно прейти при выполнении onClick
-    step = String
+    step = String(default=None, min_occurs=1)
     # количество записей в таблице
-    records = String
+    records = String(default=None, min_occurs=1)
 
     def __init__(self, dv=None, briefmode=None,
                  varObj=None, head=None, body=None, values=None,
                  fields=None, onClick=None, addAction=None, step=None,
                  records=None):
         super(Table, self).__init__()
+        # print("DEBUG Table init")
+
         if dv:
             self.head = []
             self.body = []
@@ -208,11 +205,11 @@ class Table(DataVarsSerializer):
 
 
 class Option(DataVarsSerializer):
-    shortopt = String
-    longopt = String
-    metavalue = String
-    syntax = String
-    help = String
+    shortopt = String(default=None, min_occurs=1)
+    longopt = String(default=None, min_occurs=1)
+    metavalue = String(default=None, min_occurs=1)
+    syntax = String(default=None, min_occurs=1)
+    help = String(default=None, min_occurs=1)
 
     def __init__(self, optlist, metaval, helpval, syntax=""):
         super(Option, self).__init__()
@@ -227,20 +224,20 @@ class Option(DataVarsSerializer):
 
 
 class Field(DataVarsSerializer):
-    name = String  # varname from Datavars
-    label = String  # label for GUI
-    type = String  # data type of variable
-    opt = Option  # opt for cmdline
-    help = String  # help for cmdline (GUI?)
-    element = String  # type element
-    guitype = String # addon guitype
-    choice = Array(String)  # value (combobox,comboedit)
-    listvalue = Array(String)  # current listvalue
-    default = Boolean  # default value or False
-    value = String  # current value
+    name = String(default=None, min_occurs=1)  # varname from Datavars
+    label = String(default=None, min_occurs=1)  # label for GUI
+    type = String(default=None, min_occurs=1)  # data type of variable
+    opt = Option # opt for cmdline
+    help = String(default=None, min_occurs=1)  # help for cmdline (GUI?)
+    element = String(default=None, min_occurs=1)  # type element
+    guitype = String(default=None, min_occurs=1) # addon guitype
+    choice = Array(String, default=None, min_occurs=1)  # value (combobox,comboedit)
+    listvalue = Array(String, default=None, min_occurs=1)  # current listvalue
+    default = Boolean(default=None, min_occurs=1)  # default value or False
+    value = String(default=None, min_occurs=1)  # current value
     tablevalue = Table  # current table value
-    uncompatible = String  # message for uncompatibility variable
-    comments = Array(String)  # comments for choice
+    uncompatible = String(default=None, min_occurs=1)  # message for uncompatibility variable
+    comments = Array(String, default=None, min_occurs=1) # comments for choice
 
     def __init__(self, dv=None, varObj=None, expert=False, briefmode=False,
                  inbrief=False, groupVars=(), onlyhelp=False, **kwargs):
@@ -282,6 +279,8 @@ class Field(DataVarsSerializer):
                     self.default = not varObj.wasSet
                 self.choice, self.comments = self.getChoice(varObj)
                 if self.type == "table":
+                    # print("DEBUG_table creation")
+
                     self.tablevalue = Table(dv=dv, briefmode=briefmode,
                                             varObj=varObj)
                     if self.tablevalue.writable:
@@ -307,11 +306,11 @@ class Field(DataVarsSerializer):
 
 
 class GroupField(DataVarsSerializer):
-    name = String
-    fields = Array(Field)
-    prevlabel = String
-    nextlabel = String
-    last = Boolean
+    name = String(default=None, min_occurs=1)
+    fields = Array(Field, default=None, min_occurs=1)
+    prevlabel = String(default=None, min_occurs=1)
+    nextlabel = String(default=None, min_occurs=1)
+    last = Boolean(default=None, min_occurs=1)
 
     def __init__(self, name="", fields=list(), prevlabel="",
                  nextlabel="", last=False, dv=None, info=None,
@@ -401,8 +400,8 @@ class GroupField(DataVarsSerializer):
 
 
 class ViewInfo(DataVarsSerializer):
-    groups = Array(GroupField)
-    has_brief = Boolean
+    groups = Array(GroupField, default=None, min_occurs=1)
+    has_brief = Boolean(default=None, min_occurs=1)
 
     def __init__(self, datavars=None, step=None, expert=None, allsteps=False,
                  brief=None, brief_label=None, has_brief=False, groups=list(),
@@ -424,6 +423,9 @@ class ViewInfo(DataVarsSerializer):
             # interate all vars group
             if step in (0, -1, None) or allsteps:
                 briefData = datavars.getBrief()
+                # print("DEBUG viewInfo")
+                # print("var groups: %s " % bool(varGroups))
+                # print(datavars)
                 self.groups.append(self.stepGroup(varGroups, brief_label,
                                                   help_value=briefData.get(
                                                       "help", None),
@@ -433,6 +435,8 @@ class ViewInfo(DataVarsSerializer):
                                                       "image", "")))
             for i, groupInfo in enumerate(varGroups):
                 if step in (None, -1) or step == i:
+                    # print("viewInfo step %s " % i)
+
                     self.groups.append(GroupField(dv=datavars, info=groupInfo,
                                                   last=(lastGroup == i),
                                                   expert=expert,
@@ -463,6 +467,9 @@ class ViewInfo(DataVarsSerializer):
     def stepGroup(self, groupInfo, brief_label, help_value=None,
                   next_value=None, image_value=""):
         """Step group"""
+        # print("DEBUG what are you doing stepGroup")
+        # print(groupInfo)
+        # print(brief_label)
         return GroupField(fields=[
             Field(name="Steps",
                   element="table",
@@ -482,28 +489,28 @@ class ViewInfo(DataVarsSerializer):
         # 'multichoice_add', 'check', 'check_tristate', 'expert', 'input']
 
 
-class ViewParams(ClassSerializer):
+class ViewParams(ComplexModel):
     """
     Struct for _view methods
     """
-    step = Integer  # number of group variables
-    expert = Boolean  # request expert variables
-    brief = Boolean  # request brief variables
-    onlyhelp = Boolean  # request params for only help
-    help_set = Boolean  # set cl_help_set to on
-    conargs = Array(String)  # set cl_console_args
-    dispatch_usenew = Boolean  # set cl_dispatch_conf to usenew
-    clienttype = String  # type of client "gui","console"
+    step = Integer(default=None, min_occurs=1)  # number of group variables
+    expert = Boolean(default=None, min_occurs=1)  # request expert variables
+    brief = Boolean(default=None, min_occurs=1)  # request brief variables
+    onlyhelp = Boolean(default=None, min_occurs=1)  # request params for only help
+    help_set = Boolean(default=None, min_occurs=1)  # set cl_help_set to on
+    conargs = Array(String, default=None, min_occurs=1)  # set cl_console_args
+    dispatch_usenew = Boolean(default=None, min_occurs=1)  # set cl_dispatch_conf to usenew
+    clienttype = String(default=None, min_occurs=1)  # type of client "gui","console"
 
 
 #########
 # MESSAGE
 #########
-class ReturnedMessage(ClassSerializer):
-    type = String
-    field = String
-    message = String
-    expert = Boolean
+class ReturnedMessage(ComplexModel):
+    type = String(default=None, min_occurs=1)
+    field = String(default=None, min_occurs=1)
+    message = String(default=None, min_occurs=1)
+    expert = Boolean(default=None, min_occurs=1)
     field_obj = Field
 
     def __init__(self, type=None, field=None, message=None,
@@ -519,13 +526,13 @@ class ReturnedMessage(ClassSerializer):
             self.field_obj = None
 
 
-class Message(ClassSerializer):
-    type = String
-    message = String
-    id = Integer
-    result = Boolean
-    onlyShow = String
-    default = String
+class Message(ComplexModel):
+    type = String(default=None, min_occurs=1)
+    message = String(default=None, min_occurs=1)
+    id = Integer(default=None, min_occurs=1)
+    result = Boolean(default=None, min_occurs=1)
+    onlyShow = String(default=None, min_occurs=1)
+    default = String(default=None, min_occurs=1)
 
     def __init__(self, type='normal', message=None, id=None,
                  result=None, onlyShow=None, default=None):
@@ -538,11 +545,11 @@ class Message(ClassSerializer):
         self.default = default
 
 
-class ReturnProgress(ClassSerializer):
-    percent = Integer
-    short_message = String
-    long_message = String
-    control = Integer
+class ReturnProgress(ComplexModel):
+    percent = Integer(default=None, min_occurs=1)
+    short_message = String(default=None, min_occurs=1)
+    long_message = String(default=None, min_occurs=1)
+    control = Integer(default=None, min_occurs=1)
 
     def __init__(self, percent=0, short_message=None, long_message=None,
                  control=None):
@@ -553,15 +560,15 @@ class ReturnProgress(ClassSerializer):
         self.control = control
 
 
-class Frame(ClassSerializer):
-    values = Array(Message)
+class Frame(ComplexModel):
+    values = Array(Message, default=None, min_occurs=1)
 
 
 # get and send client messages
 class CoreWsdl(CoreServiceInterface):
     perm_denied = [Message(type='error', message='403 Forbidden')]
-
-    def callAction(self, sid, info, logicClass=None,
+    @staticmethod
+    def callAction(cls, sid, info, logicClass=None,
                    method_name=None, actionClass=None,
                    callbackRefresh=lambda sid, dv: True,
                    invalidators=None, depend_methods=()):
@@ -570,12 +577,12 @@ class CoreWsdl(CoreServiceInterface):
         """
         if not logicClass:
             logicClass = {}
-        dv = self.get_cache(sid, method_name, "vars")
+        dv = cls.get_cache(sid, method_name, "vars")
         try:
             if not dv:
-                dv = getattr(self, "%s_vars" % method_name)()
+                dv = getattr(cls, "%s_vars" % method_name)()
             else:
-                callbackRefresh(sid, dv)
+                callbackRefresh(cls, sid, dv)
                 dv.processRefresh()
             checkonly = False
             checkall = False
@@ -604,16 +611,16 @@ class CoreWsdl(CoreServiceInterface):
             if isinstance(logicClass, dict):
                 for k, v in logicClass.items():
                     objs[k] = type("Logic", (CommonLink, v, object), {})()
-            install_meth = type("CommonCore", (self.Common,
+            install_meth = type("CommonCore", (cls.Common,
                                                actionClass, object), {})
-            pid = self.startprocess(sid, target=install_meth,
+            pid = cls.startprocess(cls, sid, target=install_meth,
                                     method="run",
                                     method_name=method_name,
                                     args_proc=(objs, dv,))
             returnmess = ReturnedMessage(type='pid', message=pid)
             returnmess.type = "pid"
             returnmess.message = pid
-            self.clear_cache(sid, method_name)
+            cls.clear_cache(sid, method_name)
             cache_cleared = [method_name]
             clear_list = list(LoadedMethods.methodDepends[method_name])
             while clear_list:
@@ -621,37 +628,43 @@ class CoreWsdl(CoreServiceInterface):
                 if method_clear_name not in cache_cleared:
                     clear_list.extend(
                         LoadedMethods.methodDepends[method_clear_name])
-                    self.clear_cache_method(method_clear_name)
+                    cls.clear_cache_method(method_clear_name)
                     cache_cleared.append(method_clear_name)
             dv = None
             return [returnmess]
         finally:
             if dv:
                 # print "Set cache", dv.Get('ur_unix_group_name_exists')
-                self.set_cache(sid, method_name, "vars", dv, smart=False)
+                cls.set_cache(sid, method_name, "vars", dv, smart=False)
 
-    def fixInstallLocalization(self, sid, dv):
+    @staticmethod
+    def fixInstallLocalization(cls, sid, dv):
         """
         Метод смены локализации интерфейса на лету (во время выбора
         параметров метода)
         """
+        # print("DEBUG fixInstallLocalization")
+        # print(dv)
         if "--start" not in sys.argv:
             return False
         import threading
 
         curThread = threading.currentThread()
         curThread.lang = dv.Get('install.os_install_locale_lang')
-        currentLang = self.get_cache(sid, "install", "lang")
+        # print("LOCALE DEBUG: thread lang: %s" % curThread.lang)
+        currentLang = cls.get_cache(sid, "install", "lang")
+        # print("LOCALE DEBUG: cached lang: %s" % currentLang)
         if currentLang != curThread.lang:
             dv.clearGroups()
-            self.install_vars(dv)
+            cls.install_vars(cls, dv)
             dv.reinit()
             return True
         else:
             return False
 
     # verification of compliance certificate and process (pid)
-    def check_cert_pid(self, sid, pid):
+    @staticmethod
+    def check_cert_pid(cls, sid, pid):
         import threading
 
         curThread = threading.currentThread()
@@ -659,18 +672,18 @@ class CoreWsdl(CoreServiceInterface):
 
         from cert_cmd import find_cert_id
 
-        cert_id = find_cert_id(cert, self.data_path, self.certbase)
+        cert_id = find_cert_id(cert, cls.data_path, cls.certbase)
         cert_id = int(cert_id)
         if cert_id == 0:
             return 0
 
         # session file
-        if not os.path.exists(self.sids):
-            os.system('mkdir %s' % self.sids)
+        if not os.path.exists(cls.sids):
+            os.system('mkdir %s' % cls.sids)
 
         check = 0
         try:
-            fd = open(self.sids_file, 'r')
+            fd = open(cls.sids_file, 'r')
         except IOError:
             return 0
         while 1:
@@ -686,7 +699,7 @@ class CoreWsdl(CoreServiceInterface):
 
         if check == 0:
             return 0
-        fd = open(self.sids_pids, 'r')
+        fd = open(cls.sids_pids, 'r')
         while 1:
             try:
                 # read out on 1 record
@@ -701,77 +714,59 @@ class CoreWsdl(CoreServiceInterface):
         return 0
 
     # send to client all new message
-    def process_messages(self, pid, client_type):
+    @staticmethod
+    def process_messages(cls, pid, client_type):
         result = []
-        while len(self.glob_frame_list[pid]) > \
-                self.glob_process_dict[pid]['counter']:
-            item = self.glob_process_dict[pid]['counter']
-            only_show = self.glob_frame_list[pid][item].onlyShow
+        while len(cls.glob_frame_list[pid]) > \
+                cls.glob_process_dict[pid]['counter']:
+            item = cls.glob_process_dict[pid]['counter']
+            only_show = cls.glob_frame_list[pid][item].onlyShow
             if (not client_type or
                     not only_show or only_show == client_type):
-                result.append(self.glob_frame_list[pid][item])
-            self.glob_process_dict[pid]['counter'] += 1
+                result.append(cls.glob_frame_list[pid][item])
+            cls.glob_process_dict[pid]['counter'] += 1
         return result
 
     # send to client new message from frame
-    def client_get_frame(self, sid, pid, client_type):
-        if self.check_cert_pid(sid, pid):
-            return self.process_messages(pid, client_type)
-        return self.perm_denied
+    @staticmethod
+    def client_get_frame(cls, sid, pid, client_type):
+        if cls.check_cert_pid(cls, sid, pid):
+            return cls.process_messages(cls, pid, client_type)
+        return CoreWsdl.perm_denied
 
     # send to client new message from frame
-    def client_get_entire_frame(self, sid, pid):
-        if self.check_cert_pid(sid, pid):
+    @staticmethod
+    def client_get_entire_frame(cls, sid, pid):
+        if cls.check_cert_pid(cls, sid, pid):
             try:
-                results = self.glob_frame_list[pid]
+                results = cls.glob_frame_list[pid]
             except (AttributeError, KeyError, IndexError):
-                return self.perm_denied
-            len_glob_frame_list = len(self.glob_frame_list[pid])
-            self.glob_process_dict[pid]['counter'] = len_glob_frame_list
+                return CoreWsdl.perm_denied
+            len_glob_frame_list = len(cls.glob_frame_list[pid])
+            cls.glob_process_dict[pid]['counter'] = len_glob_frame_list
             return results
-        return self.perm_denied
+        return CoreWsdl.perm_denied
 
-    def client_get_table(self, sid, pid, id):
-        if self.check_cert_pid(sid, pid):
-            return self.glob_table_dict[pid][id]
-        return self.perm_denied
+    @staticmethod
+    def client_get_table(cls, sid, pid, id):
+        if cls.check_cert_pid(cls, sid, pid):
+            return cls.glob_table_dict[pid][id]
+        return CoreWsdl.perm_denied
 
-    def client_get_progress(self, sid, pid, id):
-        if self.check_cert_pid(sid, pid):
-            return self.glob_progress_dict[pid][id]
+    @staticmethod
+    def client_get_progress(cls, sid, pid, id):
+        if cls.check_cert_pid(cls, sid, pid):
+            return cls.glob_progress_dict[pid][id]
         return ReturnProgress(0, control=403)
-
+    
+    @staticmethod
     # get message from client
-    def client_send_message(self, sid, pid, text):
-        if self.check_cert_pid(sid, pid):
-            self.glob_process_dict[pid]['answer'] = text
+    def client_send_message(cls, sid, pid, text):
+        if cls.check_cert_pid(cls, sid, pid):
+            cls.glob_process_dict[pid]['answer'] = text
             return Message(type='normal', message="")
-        return self.perm_denied
+        return CoreWsdl.perm_denied
 
-    @rpc(Integer, Integer, String, _returns=Array(Message))
-    # @Dec.check_permissions()
-    def get_frame(self, sid, pid, client_type):
-        return self.client_get_frame(sid, pid, client_type)
-
-    @rpc(Integer, Integer, _returns=Array(Message))
-    def get_entire_frame(self, sid, pid):
-        return self.client_get_entire_frame(sid, pid)
-
-    @rpc(Integer, Integer, Integer, _returns=ReturnProgress)
-    # @Dec.check_permissions()
-    def get_progress(self, sid, pid, id):
-        return self.client_get_progress(sid, pid, id)
-
-    @rpc(Integer, Integer, Integer, _returns=Table)
-    # @Dec.check_permissions()
-    def get_table(self, sid, pid, id):
-        return self.client_get_table(sid, pid, id)
-
-    # get messages, transferred from client
-    @rpc(Integer, Integer, String, _returns=Message)
-    # @Dec.check_permissions()
-    def send_message(self, sid, pid, text):
-        return self.client_send_message(sid, pid, text)
 
 
 class WsdlAdapter(object):
