@@ -17,11 +17,10 @@
 import os
 import sys
 
-from soaplib.service import rpc
-from soaplib.service import DefinitionBase
-from soaplib.serializers.primitive import String, Integer
+from spyne import rpc
+from spyne import Service
+from spyne import String, Integer, Array
 
-from soaplib.serializers.clazz import Array
 from calculate.core.server.core_interfaces import CoreServiceInterface
 from calculate.lib.utils.files import readFile
 
@@ -35,13 +34,17 @@ __ = getLazyLocalTranslate(_)
 from clean import sid_monitor, monitor
 from tasks import restart, dbus_stop
 from loaded_methods import LoadedMethods
+from api_types import Message, ReturnProgress, Table
 import post_cert
 import post_request
 import send_cert
 
+String.Attributes.encoding = "utf-8"
 
-class Basic(DefinitionBase, CoreServiceInterface):
+# class Basic(CoreServiceInterface):
+class Basic(Service, CoreServiceInterface):
     """ Basic server class """
+   
     SERV_VERS = 0.11
     glob_process_dict = {}
     process_pid = {}
@@ -60,66 +63,78 @@ class Basic(DefinitionBase, CoreServiceInterface):
     ssl_private_key = None
     cachedict = {}
 
+    #workaround: Spyne won't let us use self ref in @rpc, 
+    # and ctx leads to original service class (Basic in this case)
+    # but some methods are gathered from CoreWsdl classes, so we need
+    # a ref to combined class
+    comb_class_ref = None
+
     # function getting object from cache
-    def get_cache(self, sid, meth_name, obj_name):
-        if sid in self.cachedict:
-            if meth_name in self.cachedict[sid]:
-                if obj_name in self.cachedict[sid][meth_name]:
-                    return self.cachedict[sid][meth_name][obj_name]
+    @staticmethod
+    def get_cache(sid, meth_name, obj_name):
+        if sid in Basic.cachedict:
+            if meth_name in Basic.cachedict[sid]:
+                if obj_name in Basic.cachedict[sid][meth_name]:
+                    return Basic.cachedict[sid][meth_name][obj_name]
         return None
 
     # function placing object in cache
-    def set_cache(self, sid, meth_name, obj_name, obj, smart=True):
+    @staticmethod
+    def set_cache(sid, meth_name, obj_name, obj, smart=True):
         try:
-            if sid not in self.cachedict:
-                self.cachedict[sid] = {}
-            if meth_name not in self.cachedict[sid]:
-                self.cachedict[sid][meth_name] = {}
-            if obj_name not in self.cachedict[sid][meth_name]:
-                self.cachedict[sid][meth_name][obj_name] = obj
+            if sid not in Basic.cachedict:
+                Basic.cachedict[sid] = {}
+            if meth_name not in Basic.cachedict[sid]:
+                Basic.cachedict[sid][meth_name] = {}
+            if obj_name not in Basic.cachedict[sid][meth_name]:
+                Basic.cachedict[sid][meth_name][obj_name] = obj
                 return True
             if smart:
                 for var_name, var_value in obj.__dict__.viewitems():
                     if var_value is not None:
-                        setattr(self.cachedict[sid][meth_name][obj_name],
+                        setattr(Basic.cachedict[sid][meth_name][obj_name],
                                 var_name, var_value)
             else:
-                self.cachedict[sid][meth_name][obj_name] = obj
+                Basic.cachedict[sid][meth_name][obj_name] = obj
             return True
         except Exception as e:
             if isinstance(e, SyntaxError):
                 raise
             return False
 
-    def clear_cache(self, sid, meth_name=None, obj_name=None):
-        if sid not in self.cachedict:
+   
+
+    @staticmethod
+    def clear_cache(sid, meth_name=None, obj_name=None):
+        if sid not in Basic.cachedict:
             return True
         if meth_name:
-            if meth_name not in self.cachedict[sid]:
+            if meth_name not in Basic.cachedict[sid]:
                 return True
 
             if obj_name:
-                if obj_name not in self.cachedict[sid][meth_name]:
+                if obj_name not in Basic.cachedict[sid][meth_name]:
                     return True
                 else:
-                    obj = self.cachedict[sid][meth_name].pop(obj_name, None)
+                    obj = Basic.cachedict[sid][meth_name].pop(obj_name, None)
                     if hasattr(obj, 'close'):
                         obj.close()
             else:
-                method_dict = self.cachedict[sid].pop(meth_name, None)
+                method_dict = Basic.cachedict[sid].pop(meth_name, None)
                 for val_obj in method_dict.values():
                     if hasattr(val_obj, 'close'):
                         val_obj.close()
         else:
-            session_dict = self.cachedict.pop(sid, None)
+            session_dict = Basic.cachedict.pop(sid, None)
             for method_dict in session_dict.values():
                 for val_obj in method_dict.values():
                     if hasattr(val_obj, 'close'):
                         val_obj.close()
 
-    def clear_cache_method(self, method=None):
-        for k, v in self.cachedict.items():
-            self.clear_cache(k, method)
+    @staticmethod
+    def clear_cache_method(method=None):
+        for k, v in Basic.cachedict.items():
+            Basic.clear_cache(k, method)
 
     @staticmethod
     def set_paths(data_path, certbase, serv_certbase, rights,
@@ -144,7 +159,13 @@ class Basic(DefinitionBase, CoreServiceInterface):
         Basic.max_pid = int(max_pid)
         Basic.log_filename = log_filename
 
-    def run_tasks(self):
+
+    @staticmethod
+    def set_comb_class_ref(comb_class_ref):
+        Basic.comb_class_ref = comb_class_ref
+
+    @staticmethod
+    def run_tasks():
         """
         Запуситить регулярные задачи
         """
@@ -153,11 +174,11 @@ class Basic(DefinitionBase, CoreServiceInterface):
         monitoring = threading.Thread(target=monitor,
                                       args=(Basic.certbase, Basic.sids_file))
         sid_mon = threading.Thread(target=sid_monitor,
-                                   args=(Basic.sids_file, Basic.sids, self))
+                                   args=(Basic.sids_file, Basic.sids, Basic.comb_class_ref))
         restart_watchdog = threading.Thread(target=restart,
-                                            args=(self.glob_process_dict,))
+                                            args=(Basic.comb_class_ref.glob_process_dict,))
         dbus_stop_mon = threading.Thread(target=dbus_stop,
-                                     args=(self.glob_process_dict, Basic.sids, self))
+                                     args=(Basic.comb_class_ref.glob_process_dict, Basic.comb_class_ref.sids, Basic.comb_class_ref))
         threads = []
         for thread, success, failed in (
                 (monitoring, _("General monitoring started"),
@@ -177,20 +198,21 @@ class Basic(DefinitionBase, CoreServiceInterface):
                 print failed
         print
 
-    def killall(self):
+    @staticmethod
+    def killall():
         sys.stdout.write('\n' + _('Closing all processes') + '...')
         sys.stdout.flush()
         import time
         # Waiting for closing
-        for pid in self.process_pid.keys():
+        for pid in Basic.process_pid.keys():
             try:
-                os.kill(self.process_pid[pid].pid, 2)
+                os.kill(Basic.process_pid[pid].pid, 2)
             except OSError:
                 pass
         while True:
             num_active_process = 0
-            for pid in self.process_pid.keys():
-                if self.process_pid[pid].is_alive():
+            for pid in Basic.process_pid.keys():
+                if Basic.process_pid[pid].is_alive():
                     num_active_process += 1
 
             if num_active_process:
@@ -200,144 +222,147 @@ class Basic(DefinitionBase, CoreServiceInterface):
                 print '\n' + _('All processes are closed.')
                 return 0
             time.sleep(0.5)
+ 
 
     @rpc(_returns=Array(Integer))
-    def post_cert(self):
+    def post_cert(ctx):
         # import post_cert
-        returns = post_cert.serv_post_cert(self)
+        returns = post_cert.serv_post_cert(Basic.comb_class_ref)
         return returns
 
     @rpc(Integer, _returns=Integer)
-    def clear_session_cache(self, sid):
-        check_sid = self.check_sid_cert(sid)
+    def clear_session_cache(ctx, sid):
+        check_sid = Basic.comb_class_ref.check_sid_cert(Basic.comb_class_ref, sid)
         if not check_sid:
             return 1
         # clear cache
-        self.clear_cache(sid)
+        Basic.clear_cache(sid)
         return 0
 
     @rpc(Integer, String, _returns=Integer)
-    def clear_method_cache(self, sid, method_name):
-        check_sid = self.check_sid_cert(sid)
+    def clear_method_cache(ctx, sid, method_name):
+        check_sid = Basic.comb_class_ref.check_sid_cert(Basic.comb_class_ref, sid)
         if not check_sid:
             return 1
         # clear cache
-        self.clear_cache(sid, method_name)
+        Basic.clear_cache(sid, method_name)
         return 0
 
     @rpc(Integer, Integer, _returns=Integer)
-    def clear_pid_cache(self, sid, pid):
-        if not self.check_sid_cert(sid):
+    def clear_pid_cache(ctx, sid, pid):
+        if not Basic.comb_class_ref.check_sid_cert(Basic.comb_class_ref, sid):
             return 1
 
-        if pid in self.find_sid_pid_file(sid):
+        if pid in Basic.comb_class_ref.find_sid_pid_file(Basic.comb_class_ref, sid):
             # clear pid cache
-            self._delete_pid(sid, pid)
+            Basic.comb_class_ref.delete_pid(Basic.comb_class_ref, sid, pid)
             return 0
 
         return 2
 
     @rpc(Integer, Integer, String, _returns=Array(Integer))
     def post_sid(self, sid, cert_id, lang):
-        return self.sid_cmp(sid, cert_id, lang)
+        return Basic.comb_class_ref.sid_cmp(Basic.comb_class_ref, sid, cert_id, lang)
 
     @rpc(Integer, String, _returns=(Array(Integer), Array(Integer)))
-    def init_session(self, sid, lang):
-        return self.serv_init_session(sid, lang)
+    def init_session(ctx, sid, lang):
+        return Basic.comb_class_ref.serv_init_session(Basic.comb_class_ref, sid, lang)
 
     @rpc(Integer, _returns=Array(String))
     # @Dec.check_permissions(['del_sid'])
     def del_sid(self, sid):
-        flag = self.del_sid_pid(sid)
-        self.clear_cache(int(sid))
+        flag = Basic.comb_class_ref.del_sid_pid(Basic.comb_class_ref, sid)
+        Basic.comb_class_ref.clear_cache(int(sid))
         if not flag:
-            return self.del_sid_from_file(sid)
+            return Basic.comb_class_ref.del_sid_from_file(Basic.comb_class_ref,
+                 sid)
         else:
             return ['-1']
 
     @rpc(Integer, Integer, _returns=Integer)
-    def pid_kill(self, pid, sid):
-        return self.serv_pid_kill(pid, sid)
+    def pid_kill(ctx, pid, sid):
+        return Basic.comb_class_ref.serv_pid_kill(Basic.comb_class_ref, pid, sid)
 
     @rpc(Integer, _returns=Array(Integer))
-    def list_pid(self, sid):
-        return self.find_sid_pid_file(sid)
+    def list_pid(ctx, sid):
+        return Basic.comb_class_ref.find_sid_pid_file(Basic.comb_class_ref, sid)
 
     @rpc(Integer, String, _returns=Array(Array(String)))
-    def get_methods(self, sid, client_type):
+    def get_methods(ctx, sid, client_type):
         return map(lambda x: map(str, x),
-                   self.serv_get_methods(client_type))
+                   Basic.comb_class_ref.serv_get_methods(Basic.comb_class_ref, client_type))
 
     @rpc(Integer, _returns=Array(String))
     @LoadedMethods.core_method(rights=["get-sessions"])
     # @Dec.console('list-session')
-    def get_sessions(self, sid):
-        if not self.check_sid_cert(sid):
+    def get_sessions(ctx, sid):
+        if not Basic.comb_class_ref.check_sid_cert(Basic.comb_class_ref, sid):
             return ['']
-        return self.serv_get_sessions()
+        return Basic.comb_class_ref.serv_get_sessions(Basic.comb_class_ref)
 
     @rpc(Integer, Integer, _returns=Array(String))
     # @Dec.check_permissions(["pid_info"])
-    def pid_info(self, sid, pid):
-        return self.serv_pid_info(sid, pid)
+    def pid_info(ctx, sid, pid):
+        return Basic.comb_class_ref.serv_pid_info(Basic.comb_class_ref, sid, pid)
 
     @rpc(Integer, _returns=Array(String))
     @LoadedMethods.core_method(rights=["session_info"])
-    def sid_info(self, sid):
-        return self.serv_sid_info(sid)
+    def sid_info(ctx, sid):
+        return Basic.comb_class_ref.serv_sid_info(Basic.comb_class_ref, sid)
 
     @rpc(Integer, String, _returns=Array(String))
     @LoadedMethods.check_permissions(["view_cert_right"])
     # @Dec.console('view-cert-right')
     # @Dec.gui('System')
-    def view_cert_right(self, cert_id, client_type):
-        return self.serv_view_cert_right(cert_id, self.data_path, client_type)
+    def view_cert_right(ctx, cert_id, client_type):
+        return Basic.comb_class_ref.serv_view_cert_right(Basic.comb_class_ref,
+        cert_id, Basic.data_path, client_type)
 
     @rpc(Integer, _returns=Integer)
-    def active_client(self, sid):
-        return self.active_clients(sid)
+    def active_client(ctx, sid):
+        return Basic.comb_class_ref.active_clients(Basic.comb_class_ref, sid)
 
     @rpc(String, String, String, String, _returns=String)
-    def post_client_request(self, request, ip, mac, client_type):
-        res = post_request.serv_post_client_request(request, self.data_path,
+    def post_client_request(ctx, request, ip, mac, client_type):
+        res = post_request.serv_post_client_request(request, Basic.data_path,
                                                     ip, mac, client_type,
-                                                    self.certbase,
-                                                    self.cert_path)
+                                                    Basic.certbase,
+                                                    Basic.cert_path)
         return res
 
     @rpc(String, String, _returns=Array(String))
-    def get_client_cert(self, req_id, request):
+    def get_client_cert(ctx, req_id, request):
         res = post_request.serv_get_client_cert(req_id, request,
-                                                self.data_path, self.certbase,
-                                                self.cert_path)
+                                                Basic.data_path, Basic.certbase,
+                                                Basic.cert_path)
         return res
 
     @rpc(String, String, String, _returns=String)
-    def post_server_request(self, request, ip, mac):
-        res = post_request.serv_post_server_request(request, self.data_path,
-                                                    ip, mac, self.serv_certbase,
-                                                    self.cert_path)
+    def post_server_request(ctx, request, ip, mac):
+        res = post_request.serv_post_server_request(request, Basic.data_path,
+                                                    ip, mac, Basic.serv_certbase,
+                                                    Basic.cert_path)
         return res
 
     @rpc(String, String, _returns=Array(String))
-    def get_server_cert(self, req_id, request):
+    def get_server_cert(ctx, req_id, request):
         res = post_request.serv_get_server_request(req_id, request,
-                                                   self.data_path,
-                                                   self.serv_certbase,
-                                                   self.cert_path)
+                                                   Basic.data_path,
+                                                   Basic.serv_certbase,
+                                                   Basic.cert_path)
         return res
 
     @rpc(_returns=String)
     def get_crl(self):
-        if os.path.exists(self.data_path + '/server_certs/ca.crl'):
-            return readFile(self.data_path + '/server_certs/ca.crl')
+        if os.path.exists(Basic.data_path + '/server_certs/ca.crl'):
+            return readFile(Basic.data_path + '/server_certs/ca.crl')
         return ' '
 
     @rpc(_returns=String)
-    def get_server_host_name(self):
+    def get_server_host_name(ctx):
         import OpenSSL
 
-        cert = readFile(self.ssl_certificate)
+        cert = readFile(Basic.ssl_certificate)
         cert_obj = OpenSSL.crypto.load_certificate(
             OpenSSL.SSL.FILETYPE_PEM, cert)
         subject = cert_obj.get_subject().get_components()
@@ -347,5 +372,32 @@ class Basic(DefinitionBase, CoreServiceInterface):
         return ''
 
     @rpc(_returns=String)
-    def get_ca(self):
-        return send_cert.get_ca(self.cert_path)
+    def get_ca(ctx):
+        return send_cert.get_ca(Basic.cert_path)
+
+    #moved here from api_types:
+
+    @rpc(Integer, Integer, String, _returns=Array(Message))
+    # @Dec.check_permissions()
+    def get_frame(ctx, sid, pid, client_type):
+        return Basic.comb_class_ref.client_get_frame(Basic.comb_class_ref, sid, pid, client_type)
+
+    @rpc(Integer, Integer, _returns=Array(Message))
+    def get_entire_frame(self, sid, pid):
+        return Basic.comb_class_ref.client_get_entire_frame(Basic.comb_class_ref, sid, pid)
+
+    @rpc(Integer, Integer, Integer, _returns=ReturnProgress)
+    # @Dec.check_permissions()
+    def get_progress(self, sid, pid, id):
+        return Basic.comb_class_ref.client_get_progress(Basic.comb_class_ref, sid, pid, id)
+
+    @rpc(Integer, Integer, Integer, _returns=Table)
+    # @Dec.check_permissions()
+    def get_table(self, sid, pid, id):
+        return Basic.comb_class_ref.client_get_table(Basic.comb_class_ref, sid, pid, id)
+
+    # get messages, transferred from client
+    @rpc(Integer, Integer, String, _returns=Message)
+    # @Dec.check_permissions()
+    def send_message(self, sid, pid, text):
+        return Basic.comb_class_ref.client_send_message(Basic.comb_class_ref, sid, pid, text)
