@@ -170,7 +170,8 @@ def check_server_certificate(cert, key, cert_path, args, port, auto=False):
     elif args.host:
         url = "https://%s:%d/?wsdl" % (args.host, port)
         print url + '\n' + _("connecting...")
-        from sudsds.client import Client
+        # from sudsds.client import Client
+        from suds.client import Client
         from client_class import HTTPSClientsCertTransport
         from urllib2 import URLError
 
@@ -216,7 +217,8 @@ def check_server_certificate(cert, key, cert_path, args, port, auto=False):
         url = "https://%s:%d/?wsdl" % (args.root_host, port)
         print url + '\n' + _("connecting...")
 
-        from sudsds.client import Client
+        # from sudsds.client import Client
+        from suds.client import Client
         from client_class import HTTPSClientsCertTransport
 
         try:
