@@ -23,6 +23,15 @@ from calculate.core.server.func import clearDataVars
 
 from calculate.lib.cl_lang import setLocalTranslate
 
+# from spyne.protocol.http import HttpRpc
+# from spyne.protocol.xml import XmlDocument
+# from spyne.protocol.json import JsonDocument
+from spyne.protocol.soap import Soap11
+from spyne import Application
+# from spyne.server.wsgi import WsgiApplication
+
+from spyne_adapter import CoreInnerWsdl, make_service
+
 _ = lambda x: x
 setLocalTranslate('cl_core3', sys.modules[__name__])
 
@@ -102,7 +111,7 @@ def main(*args, **keywords):
         import logging
         import logging.handlers
 
-        logging.raiseExceptions = 0
+        # logging.raiseExceptions = 0
 
         log_path = args.log_path if args.log_path else log_path_var
 
@@ -122,7 +131,7 @@ def main(*args, **keywords):
             # debug
             if args.debug:
                 logging.basicConfig(level=logging.DEBUG)
-                logger = logging.getLogger('soaplib.wsgi')
+                logger = logging.getLogger('spyne.server.wsgi')
                 logger.setLevel(logging.DEBUG)
         except IOError:
             pass
@@ -218,57 +227,73 @@ def main(*args, **keywords):
     # importing other modules
     from func import initialization
 
-    other_mod = initialization(cl_wsdl)
-    # try:
+    outer_wsdl_classes = initialization(cl_wsdl)
+    
     pack = "calculate.core.server"
     import importlib
 
     func_metaclass = importlib.import_module('%s.func_metaclass' % pack)
-
-    other_mod.append(func_metaclass.Func_MetaClass)
+    core_wsdl_classes = []
+    core_wsdl_classes.append(func_metaclass.Func_MetaClass)
 
     from calculate.core.server.baseClass import Basic
-
-    other_mod.append(Basic)
-    other_mod.append(object)
     # make server metaclass
     if args.method or args.list_methods:
         from local_call import local_method, LocalCall
 
-        CombinedServerClass = type("CombinedServerClass",
-                                   tuple([LocalCall] + other_mod), {})
-        return local_method(CombinedServerClass, args, unknown_args)
+        ClService = CoreInnerWsdl("ClService",
+                                   tuple([LocalCall] + outer_wsdl_classes + [Basic] + core_wsdl_classes), 
+                                   {
+                                        "__metaclass__" : CoreInnerWsdl
+                                   })
+        tc = ClService()
+        tc.set_comb_class_ref(tc)
+        return local_method(tc, args, unknown_args)
 
-    CombinedServerClass = type("CombinedServerClass", tuple(other_mod), {})
+    ClService = make_service(Basic, core_wsdl_classes, outer_wsdl_classes, "ClService")
 
     from server_class import ClApplication, OpenSSLAdapter
-
-    App = ClApplication([CombinedServerClass], 'tns', log=file_logger)
-    tc = App.get_service(CombinedServerClass)
-
+    #do we even need this anymore?
+    tc = ClService() 
+    App = Application([ClService], 'tns',
+        name="ClApplication",
+        in_protocol=Soap11(),
+        out_protocol=Soap11(),
+        )
     # delete all sid and pid informations file
+    wsgi_application = ClApplication(App, log=file_logger)
     cert_cmd.create_path(data_path, certbase, rights, group_rights,
                          local_data_path)
 
     # set all path
-    tc.set_paths(data_path, certbase, serv_certbase, rights, group_rights,
-                 sids, pids, sids_pids, sids_file, pids_file, max_sid,
-                 max_pid, cert_path, log_filename, cert, key)
-    # запустить выполнение регулярных задач (мониторинг и т.д.)
+    tc.set_paths(data_path, certbase, serv_certbase,
+                 rights, group_rights, sids, pids, sids_pids, 
+                 sids_file, pids_file, max_sid,
+                 max_pid, cert_path, log_filename, 
+                 cert, key)
+
+    tc.set_comb_class_ref(tc)
     tc.run_tasks()
 
+    
+
     max_num = 99999999
     import calculate.contrib
     from cherrypy.wsgiserver import CherryPyWSGIServer, WSGIPathInfoDispatcher
 
-    dispatcher = WSGIPathInfoDispatcher({'': App})
+    dispatcher = WSGIPathInfoDispatcher({'': wsgi_application})
     server = CherryPyWSGIServer(('0.0.0.0', port), dispatcher,
                                 numthreads=10, max=max_num,
                                 request_queue_size=max_num)
 
+    # logger = logging.getLogger("spyne.application")
+    # logger.setLevel(0)
     print _("listening to https://0.0.0.0:%d") % port
     print _("wsdl is located at: https://0.0.0.0:%d/?wsdl") % port
 
+        
+
+
     ca_cert = cert_path + "/ca_root.crt"
     if not os.path.exists(ca_cert):
         ca_cert = None
@@ -323,7 +348,7 @@ def main(*args, **keywords):
         server.start()
     except KeyboardInterrupt:
         try:
-            tc.killall()
+            ClService.killall()
         except KeyboardInterrupt:
             pass
         print '\n' + _("Server stopped")
