@@ -17,7 +17,8 @@
 import urllib2 as u2
 import os
 import sys
-from sudsds.transport.http import HttpTransport
+# from sudsds.transport.http import HttpTransport
+from suds.transport.http import HttpTransport
 from httplib import HTTPConnection, HTTPSConnection
 import socket
 from calculate.core.datavars import DataVarsCore
