@@ -43,9 +43,8 @@ from calculate.lib.datavars import DataVarsError, CriticalError, Variable
 from calculate.lib.utils.content import getCfgFiles
 from itertools import *
 
-from soaplib.serializers.primitive import String, Integer, Boolean
-from soaplib.serializers.clazz import Array
-from soaplib.service import rpc
+from spyne import String, Integer, Boolean, Array
+
 from calculate.core.server.api_types import ReturnedMessage, CommonInfo
 from calculate.core.server.api_types import (Field,
                                              GroupField, ViewInfo, ViewParams)
@@ -811,21 +810,22 @@ class Action(MethodsInterface):
             return False
         return True
 
-
-def commonView(self, sid, params, arg):
-    dv = self.get_cache(sid, arg, "vars")
-    if not dv:
-        dv = getattr(self, "%s_vars" % arg)()
-    else:
-        dv.processRefresh()
-    view = ViewInfo(dv, viewparams=params)
-    self.set_cache(sid, arg, "vars", dv, smart=False)
-    return view
+# Never used. Why do we need it?
+# def commonView(self, sid, params, arg):
+#     dv = self.get_cache(sid, arg, "vars")
+#     if not dv:
+#         dv = getattr(self, "%s_vars" % arg)()
+#     else:
+#         dv.processRefresh()
+#     view = ViewInfo(dv, viewparams=params)
+#     self.set_cache(sid, arg, "vars", dv, smart=False)
+#     return view
 
 
 def catchExcept(*skipException):
     class wrapper:
-        def __init__(self, f):
+        def __init__(self, f_static):
+            f = f_static.__func__
             self.f = f
             self.func_name = f.func_name
             self.func_code = f.func_code
@@ -894,25 +894,28 @@ class ActiveClientStatus(object):
 
 class CoreWsdl(CoreServiceInterface):
     # client signals about presence
-    def active_clients(self, sid):
+    @staticmethod
+    def active_clients(cls, sid):
         # curThread = threading.currentThread()
         #        REMOTE_ADDR = curThread.REMOTE_ADDR
-        self.get_lang(sid, "from active clients")
-        if 0 < sid < self.max_sid:
+
+        #why is this here?
+        # cls.get_lang(cls, sid, "from active clients")
+        if 0 < sid < cls.max_sid:
             try:
                 # open file its session
-                sid_file = self.sids + "/%d.sid" % sid
+                sid_file = cls.sids + "/%d.sid" % sid
                 if not os.path.isfile(sid_file):
                     return ActiveClientStatus.Failed
                 # check sid in sid.db
-                if not (os.path.isfile(self.sids_file) and
-                            self.find_sid_in_file(sid)):
+                if not (os.path.isfile(cls.sids_file) and
+                            cls.find_sid_in_file(cls, sid)):
                     try:
                         os.unlink(sid_file)
                     except (OSError, IOError):
                         pass
                     return ActiveClientStatus.Failed
-                with self.sid_locker:
+                with cls.sid_locker:
                     with open(sid_file) as fd:
                         # read information about session
                         sid_inf = pickle.load(fd)
@@ -931,17 +934,18 @@ class CoreWsdl(CoreServiceInterface):
         else:
             return ActiveClientStatus.WrongSID
 
-    def serv_get_methods(self, client_type):
+    @staticmethod
+    def serv_get_methods(cls, client_type):
         curThread = threading.currentThread()
         certificate = curThread.client_cert
         from cert_cmd import find_cert_id
 
-        cert_id = find_cert_id(certificate, self.data_path, self.certbase)
+        cert_id = find_cert_id(certificate, cls.data_path, cls.certbase)
 
-        rights = self.serv_view_cert_right(cert_id, self.data_path, client_type)
+        rights = cls.serv_view_cert_right(cert_id, cls.data_path, client_type)
         return_list = []
         if client_type == "console":
-            for meth in self.return_conMethod():
+            for meth in cls.return_conMethod():
                 right_flag = True
                 for right in LoadedMethods.rightsMethods[meth[1]]:
                     if right not in rights:
@@ -952,7 +956,7 @@ class CoreWsdl(CoreServiceInterface):
                 return [['0', '0']]
             return return_list
         else:
-            for meth in self.return_guiMethod():
+            for meth in cls.return_guiMethod():
                 right_flag = True
                 for right in LoadedMethods.rightsMethods[meth[1]]:
                     if right not in rights:
@@ -964,7 +968,8 @@ class CoreWsdl(CoreServiceInterface):
             return return_list
 
     # return a list of methods for the console as list
-    def return_conMethod(self):
+    @staticmethod
+    def return_conMethod():
         from loaded_methods import LoadedMethods
 
         results = []
@@ -976,7 +981,8 @@ class CoreWsdl(CoreServiceInterface):
         return results
 
     # return a list of methods for the GUI as list
-    def return_guiMethod(self):
+    @staticmethod
+    def return_guiMethod():
         from loaded_methods import LoadedMethods
 
         results = []
@@ -998,9 +1004,10 @@ class CoreWsdl(CoreServiceInterface):
         return results
 
     # get available sessions
-    def serv_get_sessions(self):
+    @staticmethod
+    def serv_get_sessions(cls):
         result = []
-        fd = open(self.sids_file, 'r')
+        fd = open(cls.sids_file, 'r')
         while 1:
             try:
                 # read all on one record
@@ -1013,11 +1020,12 @@ class CoreWsdl(CoreServiceInterface):
         return result
 
     # check client alive
-    def client_alive(self, sid, SIDS_DIR):
+    @staticmethod
+    def client_alive(cls, sid, SIDS_DIR):
         sid_path = SIDS_DIR + "/%d.sid" % sid
         if not os.path.isfile(sid_path):
             return 1
-        with self.sid_locker:
+        with cls.sid_locker:
             with open(sid_path) as fd:
                 # read information about session
                 sid_inf = pickle.load(fd)
@@ -1258,61 +1266,66 @@ class CoreWsdl(CoreServiceInterface):
             # sid = self.process_dict['sid']
             # self.args[sid] = collections.OrderedDict()
 
-    def startprocess(self, sid, target=None, method=None, method_name=None,
+    @staticmethod
+    def startprocess(cls, sid, target=None, method=None, method_name=None,
                      auto_delete=False, args_proc=()):
         """ start process """
-        pid = self.gen_pid()
-        self.add_sid_pid(sid, pid)
+        pid = cls.gen_pid(cls)
+        cls.add_sid_pid(cls, sid, pid)
 
         import multiprocessing
 
-        if self.manager is None:
-            self.__class__.manager = multiprocessing.Manager()
+        if cls.manager is None:
+            cls.manager = multiprocessing.Manager()
+        
+        
         # Manager for sending glob_process_dict between watcher and process
         # manager = multiprocessing.Manager()
-        self.glob_process_dict[pid] = self.manager.dict()
-        self.glob_process_dict[pid]['sid'] = sid
-        self.glob_process_dict[pid]['status'] = 0
-        self.glob_process_dict[pid]['time'] = ""
-        self.glob_process_dict[pid]['answer'] = None
-        self.glob_process_dict[pid]['name'] = ""
-        self.glob_process_dict[pid]['flag'] = 0
-        self.glob_process_dict[pid]['counter'] = 0
+        cls.glob_process_dict[pid] = cls.manager.dict()
+        cls.glob_process_dict[pid]['sid'] = sid
+        cls.glob_process_dict[pid]['status'] = 0
+        cls.glob_process_dict[pid]['time'] = ""
+        cls.glob_process_dict[pid]['answer'] = None
+        cls.glob_process_dict[pid]['name'] = ""
+        cls.glob_process_dict[pid]['flag'] = 0
+        cls.glob_process_dict[pid]['counter'] = 0
 
-        self.glob_frame_list[pid] = self.manager.list()
-        self.glob_progress_dict[pid] = self.manager.dict()
-        self.glob_table_dict[pid] = self.manager.dict()
+        cls.glob_frame_list[pid] = cls.manager.list()
+        cls.glob_progress_dict[pid] = cls.manager.dict()
+        cls.glob_table_dict[pid] = cls.manager.dict()
 
         # create object Common and send parameters
-        com = target(self.glob_process_dict[pid],
-                     self.glob_progress_dict[pid],
-                     self.glob_table_dict[pid],
-                     self.glob_frame_list[pid], pid)
+        com = target(cls.glob_process_dict[pid],
+                     cls.glob_progress_dict[pid],
+                     cls.glob_table_dict[pid],
+                     cls.glob_frame_list[pid], pid)
+
 
         if len(com.__class__.__bases__) > 1 and \
                 hasattr(com.__class__.__bases__[1], '__init__'):
             com.__class__.__bases__[1].__init__(com)
         # start helper
         com.method_name = method_name
-        p = multiprocessing.Process(target=self.target_helper,
-                                    args=(com, getattr(com, method)) +
+        p = multiprocessing.Process(target=cls.target_helper,
+                                    args=(cls, com, getattr(com, method)) +
                                          (method_name,) + args_proc)
 
-        self.process_pid[pid] = p
+        cls.process_pid[pid] = p
         p.start()
         if auto_delete:
             # start watcher (for kill process on signal)
-            watcher = threading.Thread(target=self.watcher_pid_proc,
-                                       args=(sid, pid))
+            watcher = threading.Thread(target=cls.watcher_pid_proc,
+                                       args=(cls, sid, pid))
 
             watcher.start()
         return str(pid)
 
     # wrap all method
-    def target_helper(self, com, target_proc, method_name, *args_proc):
-        if not os.path.exists(self.pids):
-            os.system('mkdir %s' % self.pids)
-        # PID_FILE  =  self.pids + '/%d.pid'%com.pid
+    @staticmethod
+    def target_helper(cls, com, target_proc, method_name, *args_proc):
+        if not os.path.exists(cls.pids):
+            os.system('mkdir %s' % cls.pids)
+        # PID_FILE  =  cls.pids + '/%d.pid'%com.pid
         import datetime
 
         dat = datetime.datetime.now()
@@ -1328,7 +1341,7 @@ class CoreWsdl(CoreServiceInterface):
         except Exception:
             result = False
             print_exc()
-            fd = open(self.log_filename, 'a')
+            fd = open(cls.log_filename, 'a')
             print_exc(file=fd)
             fd.close()
         try:
@@ -1359,12 +1372,12 @@ class CoreWsdl(CoreServiceInterface):
 
         except Exception:
             print_exc()
-            fd = open(self.log_filename, 'a')
+            fd = open(cls.log_filename, 'a')
             print_exc(file=fd)
             fd.close()
             com.endFrame()
 
-    def serv_view_cert_right(self, cert_id, data_path, client_type=None):
+    def serv_view_cert_right(cls, cert_id, data_path, client_type=None):
         """ rights for the selected certificate """
         try:
             cert_id = int(cert_id)
@@ -1390,7 +1403,7 @@ class CoreWsdl(CoreServiceInterface):
         find_flag = False
         # if group = all and not redefined group all
         if 'all' in groups_list:
-            fd = open(self.group_rights, 'r')
+            fd = open(cls.group_rights, 'r')
             t = fd.read()
             # find all in group_rights file
             for line in t.splitlines():
@@ -1402,20 +1415,20 @@ class CoreWsdl(CoreServiceInterface):
             if not find_flag:
                 result = []
                 if client_type == 'console':
-                    for meth_list in self.return_conMethod():
+                    for meth_list in cls.return_conMethod():
                         for right in LoadedMethods.rightsMethods[meth_list[1]]:
                             result.append(right)
                 else:
-                    for meth_list in self.return_guiMethod():
+                    for meth_list in cls.return_guiMethod():
                         for right in LoadedMethods.rightsMethods[meth_list[1]]:
                             result.append(right)
                 result = uniq(result)
                 results = result
 
         if 'all' not in groups_list or find_flag:
-            if not os.path.exists(self.group_rights):
-                open(self.group_rights, 'w').close()
-            with open(self.group_rights) as fd:
+            if not os.path.exists(cls.group_rights):
+                open(cls.group_rights, 'w').close()
+            with open(cls.group_rights) as fd:
                 t = fd.read()
                 for line in t.splitlines():
                     if not line:
@@ -1437,7 +1450,7 @@ class CoreWsdl(CoreServiceInterface):
         add_list_rights = []
         del_list_rights = []
 
-        with open(self.rights) as fr:
+        with open(cls.rights) as fr:
             t = fr.read()
             for line in t.splitlines():
                 words = line.split()
@@ -1465,36 +1478,59 @@ class CoreWsdl(CoreServiceInterface):
             results.append("No Methods")
         return results
 
-    def get_lang(self, sid, method_name=""):
+    @staticmethod
+    def get_lang(cls, sid, method_name=""):
         """ get clients lang """
+        # print("-------------------------------------")
+        # print("DEBUG get_lang")
+
         lang = None
-        SIDS_DIR = self.sids
-        with self.sid_locker:
+        SIDS_DIR = cls.sids
+        with cls.sid_locker:
+            
             sid_file = SIDS_DIR + "/%d.sid" % int(sid)
             if os.path.exists(sid_file):
-                fd = open(sid_file, 'r')
-                while True:
+                # fd = open(sid_file, 'r')
+
+                with open(sid_file, 'r') as fd:
                     try:
+                        # print("file exists: %s " % sid_file)
+                        # print("sid: %s" % sid)
                         list_sid = pickle.load(fd)
-                    except (IOError, KeyError, EOFError):
-                        break
-                    # if session id found
-                    if sid == list_sid[0]:
-                        fd.close()
-                        lang = list_sid[3]
-                        break
-                fd.close()
+                        list_sid_sid = int(list_sid[0])
+                        # print(list_sid)
+                        # print(list_sid_sid)
+                        # print(sid)
+                        # print(sid == list_sid_sid)
+                        # print(type(list_sid_sid))
+                        # print(type(sid))
+                        if sid == list_sid_sid:
+                            # print("SID FOUND")
+                            lang = list_sid[3]
+                    except IOError:
+                        print("some io error")
+                        pass
+                    except KeyError:
+                        print("Key error")
+                        pass
+                    except EOFError:
+                        print("EOF error")
+                        pass
             try:
+                # print(lang)
                 if lang.lower() not in ('uk', 'fr', 'ru', 'en'):
                     lang = "en"
             except AttributeError:
+                # print("Attr error")
                 lang = "en"
             import locale
 
             try:
                 lang = locale.locale_alias[lang.lower()]
+                # print("lang got from locale: %s" % lang)
             except (TypeError, AttributeError, IndexError, KeyError):
                 lang = locale.locale_alias['en']
+            # print("Setting lang: %s " % lang)
             return lang
 
 
@@ -1659,13 +1695,13 @@ class WsdlMeta(type):
         """
         Конструктор для создания метода-вызова для действия
         """
-
-        def wrapper(self, sid, info):
+        # @staticmethod
+        def wrapper(cls, sid, info):
             # костыль для локализации install
             callback_refresh = (
-                self.fixInstallLocalization
-                if kwargs['method_name'] == 'install' else lambda dv, sid: True)
-            return self.callAction(sid, info, logicClass=kwargs['logic'],
+                cls.fixInstallLocalization
+                if kwargs['method_name'] == 'install' else lambda cls, dv, sid: True)
+            return cls.callAction(cls, sid, info, logicClass=kwargs['logic'],
                                    actionClass=kwargs['action'],
                                    method_name=kwargs['method_name'],
                                    callbackRefresh=callback_refresh,
@@ -1673,6 +1709,7 @@ class WsdlMeta(type):
                                    )
 
         wrapper.func_name = kwargs['method_name']
+        wrapper = staticmethod(wrapper)
         func = LoadedMethods.core_method(category=kwargs.get('category', None),
                                          title=kwargs['title'],
                                          image=kwargs.get('image', None),
@@ -1680,16 +1717,19 @@ class WsdlMeta(type):
                                          user=kwargs.get('user', False),
                                          command=kwargs.get('command', None),
                                          rights=kwargs['rights'],
-                                         depends=kwargs.get('depends', ()))(
+                                         depends=kwargs.get('depends', ()),
+                                         static_method=True)(
             wrapper)
         if "--start" in sys.argv:
             info_obj = mcs.create_info_obj(**kwargs)
             info_class = type("%sInfo" % kwargs["method_name"], (CommonInfo,),
                               info_obj)
-            return rpc(Integer, info_class,
-                       _returns=Array(ReturnedMessage))(func)
-        else:
-            return func
+            
+            #total hack: carry over info_class for later use
+            func.__func__.info_class = info_class
+            # return func
+        # else:
+        return func
 
     @classmethod
     def modify_datavars(mcs, dv, data):
@@ -1711,9 +1751,11 @@ class WsdlMeta(type):
         """
         Конструктор для создания метода-представления
         """
-
-        def wrapper(self, sid, params):
-            dv = self.get_cache(sid, kwargs["method_name"], "vars")
+        # @staticmethod
+        def wrapper(cls, sid, params):
+            # print("DEBUG view constructor")
+            dv = cls.get_cache(sid, kwargs["method_name"], "vars")
+            # print("got dv from cache: %s " % dv)
             lang_changed = False
             if kwargs["groups"]:
                 def group(*args, **kwargs):
@@ -1725,8 +1767,9 @@ class WsdlMeta(type):
                 for gr in kwargs['groups']:
                     gr(group)
             if not dv:
-                dv = getattr(self, "%s_vars" % kwargs["method_name"])(
+                dv = getattr(cls, "%s_vars" % kwargs["method_name"])(cls,
                     params=params)
+                # print("got dv from cls: %s" % dv)
                 if hasattr(params, "clienttype"):
                     if params.clienttype == 'gui' and "guivars" in kwargs:
                         mcs.modify_datavars(dv, kwargs['guivars'])
@@ -1736,37 +1779,49 @@ class WsdlMeta(type):
             else:
                 # костыль для метода install, который меняет локализацию
                 # интрефейса в зависимости от выбранного параметра lang
+                # print("dv groups: %s" % bool(dv.getGroups()))
                 if kwargs["method_name"] == 'install':
-                    lang_changed = self.fixInstallLocalization(sid, dv)
+                    lang_changed = cls.fixInstallLocalization(cls, sid, dv)
                     lang = dv.Get('install.os_install_locale_lang')
-                    self.set_cache(sid, "install", "lang", lang, smart=False)
+                    cls.set_cache(sid, "install", "lang", lang, smart=False)
+                # print("dv before refresh groups: %s" % bool(dv.getGroups()))
                 dv.processRefresh()
+                # print("dv after refresh groups: %s" % bool(dv.getGroups()))
 
-            self.set_cache(sid, kwargs["method_name"], "vars", dv, smart=False)
+            cls.set_cache(sid, kwargs["method_name"], "vars", dv, smart=False)
             if "brief" in kwargs and "name" in kwargs['brief']:
                 brief_label = str(kwargs['brief']['name'])
             else:
                 brief_label = None
+            
             if kwargs["groups"]:
+                # print("group view info creation")
+                # print(params)
+                # print(dv)
+                # print(brief_label)
+                # print(dv.getGroups())
+                # print("dv groups: %s" % bool(dv.getGroups()))
                 view = ViewInfo(dv, viewparams=params,
                                 has_brief="brief" in kwargs,
                                 allsteps=lang_changed,
                                 brief_label=brief_label)
             else:
+                # print("creating empty viewInfo")
                 view = ViewInfo()
             return view
 
         wrapper.func_name = "%s_view" % kwargs['method_name']
-        return rpc(Integer, ViewParams, _returns=ViewInfo)(
-            catchExcept(kwargs.get("native_error", ()))(wrapper))
+        wrapper = staticmethod(wrapper)
+        
+        return catchExcept(kwargs.get("native_error", ()))(wrapper)
 
     @classmethod
     def datavars_constructor(mcs, **kwargs):
         """
         Конструктор для создания метода описания параметров
         """
-
-        def wrapper(self, dv=None, params=None):
+        @staticmethod
+        def wrapper(cls, dv=None, params=None):
             if not dv:
                 _dv = DataVars()
                 _dv.importVariables()
