@@ -14,11 +14,13 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import action
 import core
 import request
 import certificate
 import groups
 import setup_package
 import variable
+import backup
 
 section = "core"
